/***************************************************************************
 *   Copyright (C) 2003 by Gerhard Monzel                                  *
 *   none                                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qspinbox.h>
#include "t_add_files.h"
#include "t_dvd_fs.h"
#include "t_prj_item.h"

T_ADD_FILES::T_ADD_FILES(T_UI_MAIN *_ui) : T_PROC_CHAIN()
{
  ui          = _ui;
  logLabel    = ui->lb_log;
  logList     = ui->winLog->lv_log;
  mapFile     = NULL;
  ActFileList = NULL;
  ActFileItem = NULL;
  total_size_mb = 0;
  //nItems      = 0;


  //-- detail stuff --
  connect(ui->rb_detail_shrink, SIGNAL(stateChanged(int)),
         SLOT(slotShrinkChanged(int)));

  connect(ui->ed_detail_shrink, SIGNAL(returnPressed()),
         SLOT(slotShrinkValueChanged()));

  connect(ui->cb_detail_atrack, SIGNAL(activated(const QString &)),
         SLOT(slotATrackChanged(const QString &)));

  connect(ui->rb_detail_atype, SIGNAL(stateChanged(int)),
         SLOT(slotATypeChanged(int)));

  connect(ui->sb_detail_shift, SIGNAL(valueChanged(int)),
         SLOT(slotAShiftChanged(int)));

  //connect(ui->sb_detail_shift, SIGNAL(textChanged(void)),
  //       SLOT(slotAShiftChanged(void)));

  connect(ui->bt_detail_preview, SIGNAL(clicked()),
         SLOT(slotPreview()));

  connect(ui->fv_project, SIGNAL(selectionChanged()),
         SLOT(slotItemSelected()));
}


T_ADD_FILES::~T_ADD_FILES()
{
  if (mapFile) delete mapFile;
}

//==========================
//== add files to project ==
//==========================
int T_ADD_FILES::begin(void)
{
  if ( (ActFileList = (KFileItemList *)ui->op_browse->selectedItems()) != NULL )
  {
    //-- lock UI and clear log list --
    ui->setEnableUI(PROJECT_UI, false);
    logList->clear();

	//-- steart with first selected item --
	ActFileItem = ActFileList->first();

    return (ActFileItem != NULL);
  }
  return 0;
}

int T_ADD_FILES::step(void)
{
  KFileItem *it = ActFileItem;

  //-- nothing to do unless it's a local file --
  if (it->isDir() || !it->isLocalFile()) return 0;

  //QString fn = it->url().path();  // media filename
  QString cmd;

  //-- every media needs temporary a project file --
  //------------------------------------------------
  mapFile = new KTempFile(ui->config->workDir(), ".prj");

  if (mapFile)
  {
    //-- now invoke analyzing/indexing job for this media --
    //------------------------------------------------------
    cmd = QString().sprintf
    (
      "lvemkidx -g -p -l %s -i \"%s\"",
      (const char *)mapFile->name(), (const char *)it->url().path()
    );
    mapFile->close();

    resetAll();
    *this << cmd; //QStringList::split(" ", cmd) << it->url().path(); // set args;

    setUseShell(true);
    setEnvironment("PATH", ui->config->binPath());
    if (start(KProcess::NotifyOnExit, TRUE))
    {
      resume();
      return 1;
    }
  }
  return 0;
}

int T_ADD_FILES::stepDone(void)
{
  static int cnt = 0;

  //-- only valid media ... --
  //--------------------------
  if (itemType != -1)
  {
    //-- ... will be added --
    T_PRJ_ITEM *fi_dup =
      (T_PRJ_ITEM *)T_PRJ_ITEM::createItem(ui, itemType, ActFileItem, mapFile->name());

    if (itemType == TYPE_LVE && mapFile) mapFile->unlink();

    //-- if got project item ... --
    if (fi_dup)
    {
      //-- insert into list and select this as current --
      ui->fv_project->insertItem(fi_dup);
      ui->fv_project->clearSelection();
      ui->fv_project->setCurrentItem(fi_dup);
      ui->fv_project->setSelected(fi_dup, true);

      //-- update sizes in DVD filesystem frame --
      total_size_mb += fi_dup->getSize();

      cnt++;
    }
    else if (mapFile)
      mapFile->unlink();
  }

  //-- map file can be released here --
  if (mapFile)
  {
    delete (mapFile);
    mapFile = NULL;
  }

  //-- prepare next step --
  //-----------------------
  ActFileItem = ActFileList->next();

  //-- all steps done ? --
  //----------------------
  if (!ActFileItem)
  {
    //-- unlock UI --
    ui->setEnableUI(PROJECT_UI, true);

    //-- update sizes in DVD filesystem frame --
    ui->dvdFs->calcSizes(total_size_mb);

    //-- enable detail frame, if no at least 1 item --
    if (ui->fv_project->items()->count() == 1) ui->gb_detail->setEnabled(true);

    //-- show action finished message --
    logLabel->setText(QString().sprintf("%d file(s) imported", cnt));

    cnt = 0;

    return 0;
  }

  return 1;
}

//===============================
//== remove files from project ==
//===============================
void T_ADD_FILES::clearProject(bool clearAll)
{
  KFileItem *fi, *fi_nxt;
  int       cnt = 0;

  //-- loop to remove selected (all) items --
  for (fi = ui->fv_project->firstFileItem(); fi;)
  {
	fi_nxt = ui->fv_project->nextItem(fi);
	if (clearAll || ui->fv_project->isSelected(fi))
	{
	  total_size_mb -= ((T_PRJ_ITEM *)fi)->getSize();
      ui->fv_project->removeItem(fi);

	  delete fi;
	  cnt++;
    }
	fi = fi_nxt;
  }

  //-- select last --
  if (ui->fv_project->items()->count())
  {
    fi = ui->fv_project->firstFileItem();
    ui->fv_project->setCurrentItem(fi);
	ui->fv_project->setSelected(fi, true);
  }
  //-- disable detail frame, if no more items --
  else
    ui->gb_detail->setEnabled(false);

  //-- adjust sizes in DVD filesystem frame --
  ui->dvdFs->calcSizes(total_size_mb);

  //-- show end of action --
  logLabel->setText(QString().sprintf("%d file(s) removed", cnt));
}

//===========
//== SLOTS ==
//===========

void T_ADD_FILES::slotItemSelected(void)
{
  T_PRJ_ITEM *it = (T_PRJ_ITEM *)ui->fv_project->currentFileItem();
  if (it) it->showDetails();
}

void T_ADD_FILES::slotShrinkChanged(int v)
{
  T_PRJ_ITEM *it = (T_PRJ_ITEM *)ui->fv_project->currentFileItem();
  if (it)
  {
    uint size = it->getSize();

    total_size_mb -= size;
    ui->dvdFs->updateTitleSize(-1 * size, false);

	it->setShrink(v == QButton::On);
    size = it->getSize();

	total_size_mb += size;
	ui->dvdFs->updateTitleSize(size, true);
  }
}

void T_ADD_FILES::slotAShiftChanged(int v)
{
  T_PRJ_ITEM *it = (T_PRJ_ITEM *)ui->fv_project->currentFileItem();
  if (it)
  {
    it->setAShift(v);
  }
}

void T_ADD_FILES::slotATypeChanged(int s)
{
  T_PRJ_ITEM *it = (T_PRJ_ITEM *)ui->fv_project->currentFileItem();
  if (it)
  {
    it->setAType(s == QButton::On);
  }
}

void T_ADD_FILES::slotShrinkValueChanged(void)
{
  T_PRJ_ITEM *it = (T_PRJ_ITEM *)ui->fv_project->currentFileItem();
  if (it)
  {
    int v = ui->ed_detail_shrink->text().toInt();

	if (v<50)
    {
	  v = 50;
	  ui->ed_detail_shrink->setText("50");
	}

    ui->dvdFs->updateTitleSize(-1 * it->getSize(), false);
    it->setShrinkValue(v);
	ui->dvdFs->updateTitleSize(it->getSize(), true);
  }
}

void T_ADD_FILES::slotATrackChanged(const QString &t)
{
  T_PRJ_ITEM *it = (T_PRJ_ITEM *)ui->fv_project->currentFileItem();
  if (it)
  {
    it->setAudioParms(t.toInt());
  }
}

void T_ADD_FILES::slotReadData(KProcess *proc, char *buf, int len)
{
  proc = proc;

  if ( len && buf[0] != '-' )
    itemType = buf[0]-'0';
  else
    itemType = -1;

  //fprintf(stderr, "item type (%d) detected\n", ItemType);

}

void T_ADD_FILES::slotPreview(void)
{
  T_PRJ_ITEM *it = (T_PRJ_ITEM *)ui->fv_project->currentFileItem();
  if (it) it->preview();
}

#include "t_add_files.moc"


