/***************************************************************************
 *   Copyright (C) 2003 by Gerhard Monzel                                  *
 *   none                                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <stdlib.h>
#include <kapplication.h>
#include "t_dvd_fs.h"
#include "t_add_files.h"
#include "t_prj_item.h"

T_DVD_FS::T_DVD_FS(T_UI_MAIN *_ui) : T_PROC_CHAIN()
{
  ui             = _ui;
  logLabel       = ui->lb_log;
  logList        = ui->winLog->lv_log;

  titles         = 0;
  titles_size    = 0;
  new_title_size = 0;
  total_size     = 0;

  subProc      = new KProcIO();
  freeProc     = new KProcIO();
  winRemTitles = new T_UI_REM_TITLES(ui, "RemTitles", 0);
  winPreview   = new T_UI_DVDPREVIEW(ui, "DVDPreview", 0);

  connect(ui->cb_writer_cap, SIGNAL(activated(const QString &)),
         SLOT(slotSetMaxSpace(const QString &)));
  connect(ui->bt_remove_titles, SIGNAL(clicked()),
         SLOT(slotRemTitles()));
  connect(ui->bt_preview_dvd, SIGNAL(clicked()),
         SLOT(slotPreview()));
}


T_DVD_FS::~T_DVD_FS()
{
  if (subProc) delete subProc;
  if (freeProc) delete freeProc;
  if (winRemTitles) delete winRemTitles;
  if (winPreview) delete winPreview;
}

const QString &T_DVD_FS::getActDir(void)
{
  return actDir;
}

void T_DVD_FS::preview(int title)
{
  if (!title) title = titles;

  /*
  if (!subProc || subProc->isRunning()) return;

  subProc->resetAll();
  *subProc << "mplayer" << "-dvd-device" << actDir << QString("dvd://%1").arg(title);

  subProc->setEnvironment("PATH", ui->config->binPath());
  if ( !subProc->start(KProcess::DontCare) )
    logLabel->setText(QString("couldn't preview title (%1)").arg(title));
  else
    logLabel->setText(QString("preview of title (%1) started").arg(title));
  */

  QString cmd = QString().sprintf
  (
    "mplayer -really-quiet -dvd-device %s dvd://%d",
    (const char *)actDir, title
  );

  if ( system((const char *)cmd) == -1 )
      logLabel->setText(QString("couldn't preview title (%1)").arg(title));

}

void T_DVD_FS::updateTOC(void)
{
  if (subProc)
  {
    if (subProc->isRunning()) subProc->kill();

	subProc->resetAll();
	subProc->enableReadSignals(false);

	*subProc << "dvdauthor" << "-T" << "-o" << actDir;

	subProc->setEnvironment("PATH", ui->config->binPath());
    if (subProc->start(KProcess::Block, true))
    {
	  QString line;

      subProc->enableReadSignals(TRUE);
      //while ( subProc->fgets(line, TRUE) != -1 )logList->insertItem(line);
      while ( subProc->readln(line, TRUE) != -1 )logList->insertItem(line);
	  logLabel->setText(line + QString(" done."));
      subProc->enableReadSignals(false);
	}
  }
}


void T_DVD_FS::diskFree(void)
{
  ui->ed_space_free->setText("unknown");

  if (freeProc)
  {
    QString str;

    if (freeProc->isRunning()) freeProc->kill();

    freeProc->resetAll();
    freeProc->enableReadSignals(FALSE);
    *freeProc << "df" << "-h" << (const char *)actDir;

    freeProc->setEnvironment("PATH", ui->config->binPath());
    if (freeProc->start(KProcess::Block))
    {
      freeProc->enableReadSignals(TRUE);
      //while (freeProc->fgets(str, TRUE) != -1);
      while (freeProc->readln(str, TRUE) != -1);
      freeProc->enableReadSignals(FALSE);

	  QStringList sizes = QStringList::split(" ", str);
      if (sizes.count() == 6)  ui->ed_space_free->setText(sizes[3]);
	}
  }
}

void T_DVD_FS::numTitles(void)
{
  QDir dir(actDir + QString(FS_VIDEO_DIR));

  titles      = 0;
  titles_size = 0;

  if ( dir.exists() ) titles = dir.entryList(FS_IFO_FILTER).count();
  ui->ed_titles->setText(QString().setNum(titles));

  if (titles)
  {
    const QFileInfoList *lst = dir.entryInfoList();

	if (lst)
	{
	  for(QFileInfoList::iterator it = lst->begin(); it != lst->end(); ++it)
        titles_size += ((*it)->size()/(1024*1024));
	}
  }

}

void T_DVD_FS::updateTitleSize(int s, bool visible)
{
  new_title_size += s;
  if (visible) calcSizes(new_title_size);
}

void T_DVD_FS::calcSizes(int new_size)
{
  QString wc         = ui->cb_writer_cap->currentText();
  int     writer_cap = wc.remove('M').toInt();
  int     progress   = 0;

  new_title_size = new_size;
  total_size     = titles_size + new_title_size;

  if (!writer_cap) writer_cap = 1;
  progress = (int)(100.0/writer_cap*total_size+0.5);
  if (progress>110) progress = 110;

  ui->ed_space_used->setText(QString("%1M").arg(titles_size));
  ui->ed_space_new->setText(QString("%1M").arg(new_title_size));
  ui->ed_space_total->setText(QString("%1M").arg(total_size));
  ui->pb_space->setProgress( progress );

  if (titles)
  {
	ui->bt_preview_dvd->setEnabled(true);
	ui->bt_burn->setEnabled(true);
  }
  else
  {
    ui->bt_preview_dvd->setEnabled(false);
    ui->bt_burn->setEnabled(false);
  }
}

void T_DVD_FS::updateSpace(const QString *dest)
{
  if (dest) actDir = *dest;

  diskFree();
  numTitles();
  calcSizes(new_title_size);
}

void T_DVD_FS::removeAll(void)
{
  const QDir  dir(actDir + QString(FS_VIDEO_DIR));
  QStringList lst = dir.entryList("V*");

  QDir::setCurrent(dir.path());
  for (uint i=0; i<lst.count(); i++)
  {
    //fprintf(stderr, ":: %s\n", (const char *)lst[i]);
    QFile::remove(lst[i]);
  }

}

void T_DVD_FS::removeLast(void)
{
  const QDir  dir(actDir + QString(FS_VIDEO_DIR));
  QString     filter = QString().sprintf(FS_TITLE_FILTER, titles);
  QStringList lst    = dir.entryList(filter);

  QDir::setCurrent(dir.path());
  for (uint i=0; i<lst.count(); i++)
  {
    //fprintf(stderr, ":: %s\n", (const char *)lst[i]);
    QFile::remove(lst[i]);
  }
}

//===========
//== Slots ==
//===========
void T_DVD_FS::slotSetMaxSpace(const QString &max)
{
  ui->ed_space_max->setText(max);
  updateSpace();
}

void T_DVD_FS::slotRemTitles(void)
{
  if (!titles) return;

  if (winRemTitles)
  {
    winRemTitles->lb_title_no->setText(QString().setNum(titles));
    winRemTitles->ch_rem_titles->setChecked(false);

    if (winRemTitles->exec() == QDialog::Accepted)
	{
	  logList->clear();
	  if ( winRemTitles->ch_rem_titles->isChecked() || titles == 1 )
	  {
	    //-- all --
	    removeAll();
	  }
	  else
	  {
        //-- last --
		removeLast();
		updateTOC();
	  }
	  updateSpace();
	}
  }
}

void T_DVD_FS::slotPreview(void)
{
  winPreview->cb_title_no->clear();
  for (int i=1; i<= titles; i++)
  {
    winPreview->cb_title_no->insertItem(QString("%1").arg(i));
  }
  winPreview->cb_title_no->setCurrentItem(titles-1);

  if (winPreview->exec() == QDialog::Accepted)
  {
    preview(winPreview->cb_title_no->currentText().toInt());
  }
}

//====================================
//== dvdauthor sub process handling ==
//====================================

int T_DVD_FS::begin(void)
{
  if ( !(ui->fv_project->childCount()) ) return 0;

  //-- for test purposes write commands to Log window ... --
  //--------------------------------------------------------
  if (KApplication::keyboardModifiers() == 1)  // ShiftButton
  {
    KFileItem *fi;

    logList->clear();

    for (fi = ui->fv_project->firstFileItem(); fi; fi = ui->fv_project->nextItem(fi))
      logList->insertItem(((T_PRJ_ITEM *)fi)->buildCmd("- 2>/dev/null |"));

    return 0;
  }

  //-- ... or create xml file for dvdauthor --
  //------------------------------------------
  if ( T_PRJ_ITEM::writeXmlHeader((const char *)ui->rq_dest->url()) )
  {
    KFileItem *fi;

    logList->clear();

    for (fi = ui->fv_project->firstFileItem(); fi; fi = ui->fv_project->nextItem(fi))
      ((T_PRJ_ITEM *)fi)->writeXml();

    T_PRJ_ITEM::writeXmlFooter();

	ui->setEnableUI(DVD_FS_UI, false);
	return 1;
  }

  return 0;
}

int T_DVD_FS::step(void)
{
  /*
  QString cmd = QString().sprintf
  (
    "dvdauthor -o %s -x %s  2>&1 | lvefilter",
    (const char *)ui->rq_dest->url(), (const char *)ui->config->xmlName()
  );
  */
  QString cmd = QString().sprintf
  (
    "dvdauthor -x %s  2>&1 | lvefilter",
    (const char *)ui->config->xmlName()
  );

  fprintf(stderr, "command (%s)\n", (const char *)cmd);

  resetAll();
  *this << cmd; //QStringList::split(" ", cmd); // set args;

  setUseShell(true);
  setEnvironment("PATH", ui->config->binPath());
  if (start(KProcess::NotifyOnExit, TRUE))
  {
    resume();
    return 1;
  }

  return 0;
}

int T_DVD_FS::stepDone(void)
{
  ui->addFiles->clearProject(true);
  updateSpace();

  logLabel->setText(QString("add title done !"));

  ui->setEnableUI(DVD_FS_UI, true);
  return 0;
}

//====================================================================================

static QString srcDir;

T_MPEG_BUILD::T_MPEG_BUILD(T_UI_MAIN *_ui) : T_PROC_CHAIN()
{
  ui             = _ui;
  logLabel       = ui->lb_log;
  logList        = ui->winLog->lv_log;
}

T_MPEG_BUILD::~T_MPEG_BUILD()
{
}

int T_MPEG_BUILD::begin(void)
{
  actFileItem = NULL;
  srcDir      = "";

  if ( ui->fv_project->childCount() > 0 )
  {
    //-- for test purposes write commands to Log window ... --
    //--------------------------------------------------------
    if (KApplication::keyboardModifiers() == 1)  // ShiftButton
    {
      KFileItem *fi;
      QString   out;

      logList->clear();

      for (fi = ui->fv_project->firstFileItem(); fi; fi = ui->fv_project->nextItem(fi))
      {
	    out = "\"" + ui->dvdFs->getActDir() + fi->url().filename() + ".mpg\"";
        logList->insertItem(((T_PRJ_ITEM *)fi)->buildCmd((const char *)out, true));
      }
      return 0;
    }
        
    srcDir = ui->cb_browse->currentText();
    ui->slotUpdateDir(ui->dvdFs->getActDir());  

    //-- ... or lock UI and clear log list --
    ui->setEnableUI(PROJECT_UI, false);
    logList->clear();

    //-- then start with first selected item --
    actFileItem = ui->fv_project->firstFileItem();

    return (actFileItem != NULL);
  }
  
  return 0;
}

int T_MPEG_BUILD::step(void)
{
  T_PRJ_ITEM *it = (T_PRJ_ITEM *)actFileItem;

  //-- TODO test "it" whether muxing or not to set extension of final out name --
  QString out = "\"" + ui->dvdFs->getActDir() + it->url().filename() + ".mpg\"";
  QString cmd = it->buildCmd((const char *)out, true);

  //fprintf(stderr, "cmd: (%s)\n", (const char *)cmd);
  ui->fv_project->clearSelection();
  ui->fv_project->setCurrentItem(it);
  ui->fv_project->setSelected(it, true);

  //-- now invoke builder --
  //------------------------
  resetAll();
  *this << cmd; //QStringList::split(" ", cmd); // set args;

  setUseShell(true);
  setEnvironment("PATH", ui->config->binPath());
  if (start(KProcess::NotifyOnExit, TRUE))
  {
    resume();
    return 1;
  }

  return 0;
}

int T_MPEG_BUILD::stepDone(void)
{
  //-- prepare next step --
  //-----------------------
  if (actFileItem) actFileItem = ui->fv_project->nextItem(actFileItem);

  //-- all steps done ? --
  //----------------------
  if (!actFileItem)
  {
    //-- update sizes in DVD filesystem frame --
    ui->dvdFs->updateSpace();

    //-- unlock UI --
    ui->setEnableUI(PROJECT_UI, true);

    //-- show action finished message --
    logLabel->setText(QString().sprintf("all files processed"));

    if (!srcDir.isEmpty()) ui->slotUpdateDir(KURL(srcDir)); 
    
    return 0;
  }

  return 1;
}

#include "t_dvd_fs.moc"
