/***************************************************************************
 *   Copyright (C) 2003 by Gerhard Monzel                                  *
 *   none                                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef T_DVD_FS_H
#define T_DVD_FS_H

#include <qlabel.h>
#include <qprogressbar.h>
#include <qcheckbox.h>
#include "t_ui_main.h"
#include "t_ui_rem_titles.h"
#include "t_ui_dvdpreview.h"
#include "t_proc_chain.h"

#define FS_VIDEO_DIR    "VIDEO_TS/"
#define FS_AUDIO_DIR    "AUDIO_TS/"
#define FS_IFO_FILTER   "VTS_??_0.IFO"
#define FS_TITLE_FILTER "VTS_%02d_?.*"

class T_DVD_FS : public T_PROC_CHAIN
{
  Q_OBJECT

  protected:
    T_UI_MAIN       *ui;
    T_UI_REM_TITLES *winRemTitles;
    T_UI_DVDPREVIEW *winPreview;

    QString   actDir;
    int       titles;
    int       titles_size;
    int       new_title_size;
    int       total_size;

    KProcIO   *subProc;
    KProcIO   *freeProc;

  public:
    T_DVD_FS(T_UI_MAIN *_ui);

    ~T_DVD_FS();

    void updateSpace(const QString *dest = NULL);
    void calcSizes(int new_size);
    void updateTitleSize(int s, bool visible);

    const QString &getActDir(void);

  protected:
    void preview(int title=0);
    void updateTOC(void);
    void diskFree(void);
    void numTitles(void);

    void removeAll(void);
    void removeLast(void);

    virtual int begin(void);
    virtual int step(void);
    virtual int stepDone(void);

  protected slots:
    void slotPreview(void);
    void slotSetMaxSpace(const QString &max);
    void slotRemTitles(void);

};

class T_MPEG_BUILD : public T_PROC_CHAIN
{
  Q_OBJECT

  protected:
    T_UI_MAIN  *ui;
    KFileItem  *actFileItem;

  public:
    T_MPEG_BUILD(T_UI_MAIN *_ui);

    ~T_MPEG_BUILD();

  protected:
    virtual int begin(void);
    virtual int step(void);
    virtual int stepDone(void);
};

#endif
