/***************************************************************************
 *   Copyright (C) 2003 by Gerhard Monzel                                  *
 *   none                                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <stdlib.h>
#include <qiodevice.h>
#include <qspinbox.h>
#include "t_prj_item.h"
#include "t_config.h"
#include "t_dvd_fs.h"

//================
//== T_PRJ_ITEM ==
//================
T_UI_MAIN   *T_PRJ_ITEM::ui      = NULL;
QTextStream *T_PRJ_ITEM::xml_os  = NULL;
int         T_PRJ_ITEM::count    = 0;
QFile       T_PRJ_ITEM::xml_of;
QString     T_PRJ_ITEM::mainCmd;

T_PRJ_ITEM::T_PRJ_ITEM(T_UI_MAIN *_ui, KFileItem &it, QString map)
 : KFileItem(it)
{
  ui      = _ui;
  mapName = map;
  xml_os  = NULL;
  delMap  = true;

  est_size = size()/(1024*1024);
  scenes   = 1;
  pictures = 0;

  //mpeg2    = false;
  aTrack     = 0;
  forceAType = false; //true;
  aShift     = 0;

  media.n_as = 1;
  media.n_vs = 0;
  media.audio_tracks[0].sample_rate = -1;
  media.audio_tracks[0].bit_rate = -1;
  media.audio_tracks[0].id = 0;

  QFile f;
  if ( it.url().path().find(".idx") == -1 )
    f.setName(it.url().path() + QString(".idx"));
  else
    f.setName(it.url().path());

  if (f.open(IO_ReadOnly))
  {
   if (f.readBlock((char *)&media, sizeof(T_STREAM)) != -1)
    {
	  pictures = media.pict_cnt;
	  //mpeg2    = (media.mpeg2_flag == 1);
	}
    f.close();
  }

  type        = TYPE_UNKNOWN;
  typeStr     = "Unknow stream";
  cmdOpts     = "";
  shrink      = false;
  shrinkValue = 2.0;

  count++;
}

T_PRJ_ITEM::~T_PRJ_ITEM()
{
  if (xml_os) delete xml_os;

  if (delMap && !mapName.isEmpty()) QFile::remove(mapName);
  count--;

}

const QString &T_PRJ_ITEM::getTypeStr(void)
{
  return typeStr;
}

uint T_PRJ_ITEM::getSize(void)
{
  double ov = 0.0;

  //-- overhead --
  switch (media.n_as)
  {
    case 0:
    case 1:
      ov = 0.1;
	  break;

	case 2:
	  ov = 0.05;
      break;

    case 3:
	  ov = 0.03;
	  break;

	case 4:
	  ov = 0.02;
  }



  if (shrink)
  {
    uint s = (uint)(est_size / shrinkValue + 1);
	return (s + (uint)(s*ov));
  }
  else
    return (est_size + (uint)(est_size*ov));
}

//==========================
//== project item Details ==
//==========================
void T_PRJ_ITEM::showDetails(void)
{
  ui->ed_detail_type->setText(typeStr);
  ui->ed_detail_scenes->setText(QString().setNum(scenes));
  ui->ed_detail_picts->setText(QString().setNum(pictures));
  ui->ed_detail_size->setText(QString("%1M").arg(est_size));
  ui->rb_detail_shrink->setChecked(shrink);
  ui->rb_detail_atype->setChecked(forceAType);

  setShrink(shrink);
  setVideoParms();
  setAudioParms();

}

void T_PRJ_ITEM::setAShift(int v)
{
  aShift = v;
}

void T_PRJ_ITEM::setAType(bool s)
{
  forceAType = s;
  setAudioParms();
}

void T_PRJ_ITEM::setShrink(bool s)
{
  uint value = (uint)(100.0 * (1.0/shrinkValue) + 0.5);

  shrink = s;

  ui->ed_detail_shrink->setText(QString().setNum(value));
  ui->ed_detail_size->setText(QString("%1M").arg(getSize()));

  if (shrink)
    ui->ed_detail_shrink->setEnabled(true);
  else
    ui->ed_detail_shrink->setEnabled(false);
}

void T_PRJ_ITEM::setShrinkValue(int v)
{
  shrinkValue = 100.0/v;
  setShrink(shrink);
}

void T_PRJ_ITEM::setAudioParms(int t)
{
  if (t != -1) aTrack = t;

  ui->gb_detail_audio->setEnabled(true);

  ui->cb_detail_atrack->clear();
  for (int i=0; i< media.n_as; i++)
    ui->cb_detail_atrack->insertItem(QString("%1").arg(i));

  ui->cb_detail_atrack->setCurrentItem(aTrack);

  if (media.audio_tracks[aTrack].sample_rate == -1)
    ui->ed_detail_sr->setText("n/a");
  else
    ui->ed_detail_sr->setText(QString().setNum(media.audio_tracks[aTrack].sample_rate));

  if (media.audio_tracks[aTrack].bit_rate == -1)
    ui->ed_detail_br->setText("n/a");
  else
	ui->ed_detail_br->setText(QString().setNum(media.audio_tracks[aTrack].bit_rate));

  ui->sb_detail_shift->setValue(aShift);

  if (forceAType)
  {
	ui->ed_detail_atype->setText("0xC0");
  }
  else
  {
    if (media.audio_tracks[aTrack].id == 0x00)
      ui->ed_detail_atype->setText("n/a");
	else
      ui->ed_detail_atype->setText(QString().sprintf("0x%02x", media.audio_tracks[aTrack].id));
  }
}

void T_PRJ_ITEM::setVideoParms(void)
{
  static const double frtab[] = {0.0, 23.97, 24.0, 25.0, 29.97, 30.0, 50.0, 59.94, 60.0};
  static const char  *astab[] = {"n/a", "1:1", "4:3", "16:9", "2.21:1", "n/a"};

  if (media.n_vs)
  {
    double     fr  = 0.0;
    const char *as = astab[0];
    
    if (media.frame_rate >=0 && media.frame_rate<=8) fr = frtab[media.frame_rate];
    if (media.aspect >=0 && media.aspect<=5) as = astab[media.aspect];
     
    ui->gb_detail_video->setEnabled(true);
    if (!media.mpeg2_flag) ui->rb_detail_shrink->setEnabled(false);

    ui->lb_detail_mpeg->setText(QString("(mpeg%1)").arg(media.mpeg2_flag+1));
    ui->ed_detail_wxh->setText(QString().sprintf("%dx%d", media.width, media.height));
    ui->ed_detail_fr->setText(QString().setNum(fr));
    ui->ed_detail_as->setText(as);
  }
  else
  {
    ui->gb_detail_video->setEnabled(false);
  }

  ui->bt_detail_preview->setEnabled(true);
}

const QString &T_PRJ_ITEM::previewCmd(void)
{
  mainCmd = QString().sprintf
  (
    "mplayer -really-quiet \"%s\"&",
    (const char *)url().path()
  );

  return mainCmd;
}

void T_PRJ_ITEM::preview(void)
{
  system("killall mplayer");
  if (system((const char *)previewCmd()) == -1)
    fprintf(stderr, "ERROR: couldn't launch file preview !");
}

//===============
//== XML stuff ==
//===============
bool T_PRJ_ITEM::writeXmlHeader(const char *dest)
{
  bool  isOpen = false;

  if (ui)
  {
    xml_of.setName(ui->config->xmlName());
    if ( (isOpen = xml_of.open(IO_WriteOnly)) == true )
    {
      xml_os = new QTextStream((QIODevice *)&xml_of);

      *xml_os << "<dvdauthor dest=\"" << dest << "\">\n";
      *xml_os << "  <vmgm />\n";
      *xml_os << "  <titleset>\n";
      *xml_os << "    <titles>\n";
      *xml_os << "      <pgc>\n";
    }
  }

  return isOpen;
}

void T_PRJ_ITEM::writeXmlFooter(void)
{
  if (xml_os)
  {
    *xml_os << "      </pgc>\n";
    *xml_os << "    </titles>\n";
    *xml_os << "  </titleset>\n";
    *xml_os << "</dvdauthor>\n";
    delete xml_os;
  }

  xml_of.close();
  xml_os = NULL;
}

void T_PRJ_ITEM::writeXml(void)
{
  if (xml_os)
  {
    QString cmd = buildCmd("- 2>/dev/null|"); 
    
    *xml_os << XML_VOB_START;
    *xml_os << cmd.remove('"');  // final output destination
    *xml_os << XML_VOB_END;
  }
}

//========================
//== MPEG build command ==
//========================
const QString &T_PRJ_ITEM::buildCmd(const char * p, bool verbose)
{
  T_CONFIG *cfg  = ui->config;
  QString  out   = ui->dvdFs->getActDir() + url().path();
  QString  mOpts = ""; 
  
  cmdOpts = "";

  if (aTrack) cmdOpts.append(QString(" -t %1").arg(aTrack));   
  if (aShift != 0) cmdOpts.append(QString(" -sh %1").arg(aShift));
  if (!forceAType) cmdOpts.append(" -c"); 
  if (verbose) mOpts = "-2";
  
  if (!shrink)
  {
    mainCmd = QString().sprintf
    (
      "lvedump %s %s -m -s 48000 -i %s -av %s",
      (const char *)cmdOpts, (const char *)mOpts, (const char *)mapName, p 
    );
  }
  else
  {    
    mainCmd = QString().sprintf
    (
      "lvedump %s -e -s 48000 -i %s -av %s 2>/dev/null | "
      "lverequant %1.5f %s.m2v %s | "
      "lvemux -p %s -v %s -a %s.mpa -o %s",
       (const char *)cmdOpts, (const char *)mapName, (const char *)cfg->shrinkFifo(),
       shrinkValue, (const char *)cfg->shrinkFifo(), (const char *)cfg->muxFifo(),
       (const char*)mOpts ,(const char *)cfg->muxFifo(), (const char *)cfg->shrinkFifo(), p
    );
  }

  return mainCmd;
}

//===============================
//== main project item creator ==
//===============================
KFileItem *T_PRJ_ITEM::createItem(T_UI_MAIN *ui, int t, KFileItem *it, QString map)
{
  T_PRJ_ITEM *it_dup = NULL;

  switch (t)
  {
    case TYPE_LVE:
	  it_dup = (T_PRJ_ITEM *)(new T_LVE_ITEM(ui, *it, map));
      break;

    case TYPE_ES:
	  it_dup = (T_PRJ_ITEM *)(new T_ES_ITEM(ui, *it, map));
      break;

	case TYPE_WA:
	  it_dup = (T_PRJ_ITEM *)(new T_WA_ITEM(ui, *it, map));
      break;

	case TYPE_TS:
	  it_dup = (T_PRJ_ITEM *)(new T_TS_ITEM(ui, *it, map));
      break;

	case TYPE_PES:
	  it_dup = (T_PRJ_ITEM *)(new T_PES_ITEM(ui, *it, map));
      break;

	case TYPE_PVA:
	  it_dup = (T_PRJ_ITEM *)(new T_PVA_ITEM(ui, *it, map));
      break;

	case TYPE_PS:
	  it_dup = (T_PRJ_ITEM *)(new T_PS_ITEM(ui, *it, map));
      break;
  }

  return (KFileItem *)it_dup;
}


//================
//== T_LVE_ITEM ==
//================

T_LVE_ITEM::T_LVE_ITEM(T_UI_MAIN *ui, KFileItem &it, QString map) : T_PRJ_ITEM(ui, it, map)
{
  type    = TYPE_LVE;
  typeStr = "LVE project";

  mapName = it.url().path();
  delMap  = false;

  scenes   = 0;
  pictures = 0;
  est_size = 0;

  QFile       f(mapName);
  T_MEDIA_HDR hdr;

  if (f.open(IO_ReadOnly))
  {
    //fprintf(stderr, "T_LVE_ITEM (reading prj-file)\n");
    if (f.readBlock((char *)&hdr, sizeof(T_MEDIA_HDR)) != -1)
    {
      scenes   = hdr.mb_cnt;
      pictures = hdr.dist_sum;
      est_size = hdr.output_size_mb;
    }
    f.close();
  }

}

const QString &T_LVE_ITEM::previewCmd(void)
{
  QString opts = "";

  if (aShift!=0) opts.append(QString(" -sh %1").arg(aShift));

  mainCmd = QString().sprintf
  (
    "lvedump %s -M -i %s -av - | mplayer -use-stdin /dev/stdin &",
    (const char *)opts, (const char *)url().path()
  );

  return mainCmd;
}

//===============
//== T_ES_ITEM ==
//===============
T_ES_ITEM::T_ES_ITEM(T_UI_MAIN *ui, KFileItem &it, QString map) : T_PRJ_ITEM(ui, it, map)
{
  type    = TYPE_ES;
  typeStr = "Elementary Stream (ES) with ext. audio (encoded)";

  cmdOpts = "";

  QString mpa = it.url().path();
  mpa = mpa.left(mpa.findRev('.')) + ".mpa";

  est_size += (QFile(mpa).size()/(1024*1024));
}

const QString &T_ES_ITEM::buildCmd(const char *p, bool verbose)
{
  T_CONFIG *cfg      = ui->config;
  QString  videoName = url().path().replace(" ","\\ ");
  QString  audioName = videoName;
  
  audioName = audioName.left(audioName.findRev('.')) + ".mpa";

  if (verbose) cmdOpts = "-2";
      
  if (!shrink)
  {
    mainCmd = QString().sprintf
    (
      "lvemux %s -v %s -a %s -o %s",
      (const char *)cmdOpts, (const char *)videoName, (const char *)audioName, p
    );
  }
  else
  {
    mainCmd = QString().sprintf
    (
      "lverequant %1.5f %s %s | lvemux %s -v %s -a %s -o %s",
      shrinkValue, (const char *)videoName, (const char *)cfg->muxFifo(),
      (const char *)cmdOpts, (const char *)cfg->muxFifo(), (const char *)audioName, p
    );
  }

  return mainCmd;
}

//===============
//== T_WA_ITEM ==
//===============
T_WA_ITEM::T_WA_ITEM(T_UI_MAIN *ui, KFileItem &it, QString map) : T_PRJ_ITEM(ui, it, map)
{
  type    = TYPE_WA;
  typeStr = "Elemetary Stream (ES) with ext. audio (WAV)";
}

//===============
//== T_TS_ITEM ==
//===============
T_TS_ITEM::T_TS_ITEM(T_UI_MAIN *ui, KFileItem &it, QString map) : T_PRJ_ITEM(ui, it, map)
{
  type    = TYPE_TS;
  typeStr = "Transport Stream (TS)";
}

//===============
//== T_PES_ITEM ==
//===============
T_PES_ITEM::T_PES_ITEM(T_UI_MAIN *ui, KFileItem &it, QString map) : T_PRJ_ITEM(ui, it, map)
{
  type    = TYPE_PES;
  typeStr = "PES stream";
}

//===============
//== T_PVA_ITEM ==
//===============
T_PVA_ITEM::T_PVA_ITEM(T_UI_MAIN *ui, KFileItem &it, QString map) : T_PRJ_ITEM(ui, it, map)
{
  type    = TYPE_PVA;
  typeStr = "PVA stream";
}

//===============
//== T_PS_ITEM ==
//===============
T_PS_ITEM::T_PS_ITEM(T_UI_MAIN *ui, KFileItem &it, QString map) : T_PRJ_ITEM(ui, it, map)
{
  type    = TYPE_PS;
  typeStr = "Program Stream (PS)";
}

//eof
