/***************************************************************************
 *   Copyright (C) 2003 by Gerhard Monzel                                  *
 *   none                                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef T_PRJ_ITEM_H
#define T_PRJ_ITEM_H

#include <inttypes.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qradiobutton.h>
#include <kfileitem.h>
#include <kprocio.h>
#include <kprocess.h>
#include <krestrictedline.h>
#include "t_ui_main.h"

//-- same like lve/src/frame_srv.h : MPEG_xxx (don't touch) --
//------------------------------------------------------------
#define TYPE_UNKNOWN 0
#define TYPE_PS      1
#define TYPE_ES      2    // ES with MPA audio
#define TYPE_TS      3
#define TYPE_PES     4
#define TYPE_PVA     5
#define TYPE_LVE     6
#define TYPE_WA      8    // ES with separate WAV audio
#define TYPE_NA      9    // ES without audio

#define XML_VOB_START "        <vob file=\""
#define XML_VOB_END   "\" />\n"


#define MAGIC_LEN       8
#define MAX_MAGIC_LEN   8
#define LVE_MAGIC       "LVE-V1.8"   // exactly 8 chars !!!
#define MEDIA_BUF_MAGIC "LVEMEDV6"  // exactly 8 char needed !
#define EDIT_BUF_MAGIC  "LVEEDTV6"
#define MAX_ATRACKS  16

typedef struct
{
  char   magic[MAGIC_LEN+1];
  int    mb_cnt;
  int    dist_sum;
  int    output_size_mb;
} T_MEDIA_HDR;

typedef struct
{
  uint8_t id;
  int     sample_rate;
  int     bit_rate;
  int     pid;
} T_AUDIO_TRACK;

typedef struct
{
  char          magic[MAX_MAGIC_LEN];
  int8_t        type;
  int8_t        mpeg2_flag;
  int8_t        atype;
  uint8_t       vs_id;
  uint8_t       as_id;
  int           vs_pid;
  int           as_pid;
  int           pack_cnt;
  int           sys_cnt;
  int           vs_cnt;
  int           as_cnt;
  int           seq_cnt;
  int           gop_cnt;
  int           pict_cnt;

  int           width;
  int           height;
  int           aspect;
  int           frame_rate;

  char          open_gop;
  char          n_vs;
  char          n_as;

  T_AUDIO_TRACK audio_tracks[MAX_ATRACKS];
} T_STREAM;


class T_PRJ_ITEM : public KFileItem
{
  protected:
    static T_UI_MAIN *ui;

    int         type;
    QString     typeStr;

    bool        shrink;
    double      shrinkValue;

    uint        est_size;
    uint        scenes;
    uint        pictures;
    uint        aTrack;
    bool        forceAType;
    int         aShift;

    T_STREAM    media;

    bool        delMap;
    QString     mapName;

    QString     cmdOpts;

    static QFile       xml_of;
    static QTextStream *xml_os;
    static QString     mainCmd;
    static int         count;

  public:
    T_PRJ_ITEM(T_UI_MAIN *_ui, KFileItem &it, QString map);

    ~T_PRJ_ITEM();

  public:
    static KFileItem *createItem(T_UI_MAIN *ui, int t, KFileItem *it, QString map);
    static bool writeXmlHeader(const char *dest);
    static void writeXmlFooter();

    void preview(void);

    virtual void showDetails(void);
    virtual void setAShift(int v);
    virtual void setShrink(bool s);
    virtual void setAType(bool s);
    virtual void setShrinkValue(int v);
    virtual void setVideoParms(void);
    virtual void setAudioParms(int t = -1);

    virtual uint  getSize(void);
    virtual const QString &getTypeStr(void);
    virtual const QString &previewCmd(void);

    virtual void writeXml(void);
    virtual const QString &buildCmd(const char *p, bool verbose = false);
};

class T_LVE_ITEM : public T_PRJ_ITEM
{
  public:
    T_LVE_ITEM(T_UI_MAIN *ui, KFileItem &it, QString map);

  protected:
    virtual const QString &previewCmd(void);
};

class T_ES_ITEM : public T_PRJ_ITEM
{
  public:
    T_ES_ITEM(T_UI_MAIN *ui, KFileItem &it, QString map);

    virtual const QString &buildCmd(const char *p, bool verbose = false);
};

class T_WA_ITEM : public T_PRJ_ITEM
{
  public:
    T_WA_ITEM(T_UI_MAIN *ui, KFileItem &it, QString map);
};

class T_TS_ITEM : public T_PRJ_ITEM
{
  public:
    T_TS_ITEM(T_UI_MAIN *ui, KFileItem &it, QString map);
};

class T_PES_ITEM : public T_PRJ_ITEM
{
  public:
    T_PES_ITEM(T_UI_MAIN *ui, KFileItem &it, QString map);
};

class T_PVA_ITEM : public T_PRJ_ITEM
{
  public:
    T_PVA_ITEM(T_UI_MAIN *ui, KFileItem &it, QString map);
};

class T_PS_ITEM : public T_PRJ_ITEM
{
  public:
    T_PS_ITEM(T_UI_MAIN *ui, KFileItem &it, QString map);
};

#endif
