/***************************************************************************
 *   Copyright (C) 2003 by Gerhard Monzel                                  *
 *   none                                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qapplication.h>
#include "t_proc_chain.h"

//==============================
//== Constructur / Destructor ==
//==============================
T_PROC_CHAIN::T_PROC_CHAIN() : KProcIO() //KProcess()
{
  status   = 0;
  logLabel = NULL;
  logList  = NULL;

  connect((const QObject *)this, SIGNAL(processExited(KProcess *)),
                       (const QObject *)this, SLOT(slotStepDone(KProcess *)));
  connect((const QObject *)this, SIGNAL(receivedStdout(KProcess *, char *, int)),
                       (const QObject *)this, SLOT(slotReadData(KProcess *, char *, int)));
  connect((const QObject *)this, SIGNAL(readReady(KProcIO *)),
                       (const QObject *)this, SLOT(slotReadLog(KProcIO *)));
}


T_PROC_CHAIN::~T_PROC_CHAIN()
{
}

//==============================================
//== public entry Slot to start chain process ==
//==============================================
void T_PROC_CHAIN::slotBegin(void)
{
  if (begin())
  {
    status = 1;
    slotStep();
  }
  else
  {
    status = 0;
	slotStepDone(NULL);
  }
}

//===============================================
//== protected control slots no need to derive ==
//===============================================
void T_PROC_CHAIN::slotStep(void)
{
  //fprintf(stderr, "slotStep()\n");
  if (!step()) slotStepDone(NULL);
}

void T_PROC_CHAIN::slotStepDone(KProcess *proc)
{
  //fprintf(stderr, "slotStepDone()\n");
  proc = proc;

  if (stepDone())
    slotStep();
  else
    status = 0;
}

int T_PROC_CHAIN::begin(void)
{
  fprintf(stderr, "T_PROC_CHAIN::begin (1)\n");
  return 1;
}

int T_PROC_CHAIN::step(void)
{
  fprintf(stderr, "T_PROC_CHAIN::step (0)\n");
  return 0;
}

int T_PROC_CHAIN::stepDone(void)
{
  fprintf(stderr, "T_PROC_CHAIN::stepDone (0)\n");
  return 0;
}

void T_PROC_CHAIN::slotReadData(KProcess *proc, char *buf, int len)
{
  proc = proc;
  buf  = buf;
  len  = len;
  fprintf(stderr, "T_PROC_CHAIN::slotReadData\n");
}

void T_PROC_CHAIN::slotReadLog(KProcIO* proc)
{
  if (logLabel && logList)
  {
    QString line;

    proc = proc;
    while (readln(line, TRUE) != -1)
    {
      if (line.contains("WARN"))
        fprintf(stderr, "%s\n", (const char *)line);
      else
      {
        if ( !line.contains("done, estimate finish") && !line.contains("\r") )
          logList->insertItem(line);

        line.stripWhiteSpace();
        if (line.length() > 1) logLabel->setText(line);
      }
    }
  }
}

#include "t_proc_chain.moc"
