/***************************************************************************
 *   Copyright (C) 2003 by Gerhard Monzel                                  *
 *   none                                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef T_UI_MAIN_H
#define T_UI_MAIN_H

#include <qlabel.h>
#include <qcombobox.h>
#include <qptrlist.h>
#include <qvaluelist.h>
#include <qradiobutton.h>
#include <kurl.h>
#include <kfileitem.h>
#include <kdiroperator.h>
#include <kfiledetailview.h>
#include <kurlcombobox.h>
#include <kurlrequester.h>
#include <klineedit.h>
#include <kpushbutton.h>
#include <krestrictedline.h>
#include <kprocess.h>
#include <kconfig.h>
#include <kconfigdata.h>
#include "ui_main.h"
#include "t_config.h"
#include "t_ui_log.h"

#define PROJECT_UI 1
#define DVD_FS_UI  2
#define WRITER_UI  3

class T_DVD_FS;
class T_MPEG_BUILD;
class T_ADD_FILES;
class T_WRITER;

class T_PROJECT_VIEW : public KFileDetailView
{
  public:
    T_PROJECT_VIEW(QWidget *parent, const char *name);

	virtual void sort();
};

class T_UI_MAIN : public UI_MAIN
{
  Q_OBJECT

public:
  T_UI_MAIN(QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
  ~T_UI_MAIN();
  /*$PUBLIC_FUNCTIONS$*/
  void setEnableUI(int t, bool s);

public slots:
  /*$PUBLIC_SLOTS$*/
  void slotUpdateDvdFs(const QString &dest);
  void slotUpdateDir(const KURL &url);
  void slotSetDir(const KURL &url);
  void slotGetDir(void);
  void slotClearProject(void);
  void slotShowLog(void);

public:
  KDirOperator    *op_browse;
  KURLComboBox    *cb_browse;
  KFileDetailView *fv_project;
  //T_PROJECT_VIEW  *fv_project;
  T_WRITER        *writer;
  T_DVD_FS        *dvdFs;
  T_MPEG_BUILD    *mpegBuild;
  T_ADD_FILES     *addFiles;
  T_CONFIG        *config;
  T_UI_LOG        *winLog;

protected slots:
  /*$PROTECTED_SLOTS$*/
  virtual void closeEvent(QCloseEvent *e);

protected:

  void loadConfig(void);
  void saveConfig(void);

};

#endif

