/***************************************************************************
 *   Copyright (C) 2003 by Gerhard Monzel                                  *
 *   none                                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "t_writer.h"

T_WRITER::T_WRITER(T_UI_MAIN *_ui) : T_PROC_CHAIN()
{
  ui       = _ui;
  logLabel = ui->lb_log;
  logList  = ui->winLog->lv_log;

  burn = true;

  winFmtOpts = new UI_FMT_OPTS(ui, "FmtOpts", 0);
}


T_WRITER::~T_WRITER()
{
  if (winFmtOpts) delete winFmtOpts;
}

int T_WRITER::step(void)
{
  QString cmd;

  if (burn)
  {
    QString opts = "";

	if (ui->cb_writer_speed->currentItem()>0)
	  opts = "-speed=" + ui->cb_writer_speed->currentText();

    cmd = QString().sprintf
          (
		    "growisofs -use-the-force-luke %s -Z %s -dvd-video -udf %s 2>&1 | lvefilter",
            (const char *)opts,
			(const char *)ui->cb_device->currentText(),
			(const char *)ui->rq_dest->url()
		  );
  }
  else
  {
    cmd = QString().sprintf
          (
		    "dvd+rw-format %s %s 2>&1 | lvefilter",
		    (const char *)fmtOpts, (const char *)ui->cb_device->currentText()
		  );
  }

  fprintf(stderr, "command (%s)\n", (const char *)cmd);

  resetAll();
  *this << cmd; //QStringList::split(" ", cmd); // set args;

  setUseShell(true);
  setEnvironment("PATH", ui->config->binPath());
  if (start(KProcess::NotifyOnExit, TRUE))
  {
    resume();
    return 1;
  }

  return 0;
}

int T_WRITER::stepDone(void)
{
  ui->setEnableUI(WRITER_UI, true);
  return 0;
}

//== Slots ==
void T_WRITER::slotFormat(void)
{

  if (winFmtOpts)
  {
    winFmtOpts->gb_mode->setChecked(false);
    winFmtOpts->gb_area->setChecked(false);

    if (winFmtOpts->exec() == QDialog::Accepted)
    {
      burn    = false;
      fmtOpts = "";

      if (winFmtOpts->gb_mode->isChecked())
	  {
        fmtOpts += " -force";
        if (winFmtOpts->rb_force_full->isChecked())
          fmtOpts += "=full";
	  }

      if (winFmtOpts->gb_area->isChecked())
      {
	    if (winFmtOpts->rb_blank->isChecked())
          fmtOpts += " -blank";
	    else if (winFmtOpts->rb_blank_full->isChecked())
          fmtOpts += " -blank=full";
	    else if (winFmtOpts->rb_leadout->isChecked())
          fmtOpts += " -lead-out";
	  }

      ui->setEnableUI(WRITER_UI, false);

      logList->clear();
      slotBegin();
    }
  }
}

void T_WRITER::slotBurn(void)
{

  if (QMessageBox::question(ui, "klvemkdvd - Burn ? ", "Burn DVD filesystem ?", "&Yes", "&No",
                            QString::null, 0, 1) == 0)
  {
    burn = true;
    ui->setEnableUI(WRITER_UI, false);

    logList->clear();
    slotBegin();
  }
}

#include "t_writer.moc"
